@@ -130,6 +130,7 @@ extern _X_EXPORT ClientPtr clients[MAXCLIENTS];
 extern _X_EXPORT ClientPtr serverClient;
 extern _X_EXPORT int currentMaxClients;
 extern _X_EXPORT char dispatchExceptionAtReset;
+extern _X_EXPORT int terminateDelay;
 
 typedef int HWEventQueueType;
 typedef HWEventQueueType *HWEventQueuePtr;
@@ -148,6 +149,7 @@ typedef struct _TimeStamp {
 } TimeStamp;
 
 /* dispatch.c */
+extern _X_EXPORT ClientPtr GetCurrentClient(void);
 
 extern _X_EXPORT void SetInputCheck(HWEventQueuePtr /*c0 */ ,
                                     HWEventQueuePtr /*c1 */ );
@@ -240,6 +242,8 @@ extern _X_EXPORT void RemoveBlockAndWakeupHandlers(ServerBlockHandlerProcPtr blo
 
 extern _X_EXPORT void InitBlockAndWakeupHandlers(void);
 
+extern _X_EXPORT void ClearWorkQueue(void);
+
 extern _X_EXPORT void ProcessWorkQueue(void);
 
 extern _X_EXPORT void ProcessWorkQueueZombies(void);
@@ -261,13 +265,10 @@ extern _X_EXPORT Bool ClientSleep(ClientPtr client,
 extern _X_EXPORT Bool ClientSignal(ClientPtr /*client */ );
 #endif                          /* ___CLIENTSIGNAL_DEFINED___ */
 
-#ifndef ___CLIENTSIGNALALL_DEFINED___
-#define ___CLIENTSIGNALALL_DEFINED___
 #define CLIENT_SIGNAL_ANY ((void *)-1)
 extern _X_EXPORT int ClientSignalAll(ClientPtr /*client*/,
                                      ClientSleepProcPtr /*function*/,
                                      void * /*closure*/);
-#endif                          /* ___CLIENTSIGNALALL_DEFINED___ */
 
 extern _X_EXPORT void ClientWakeup(ClientPtr /*client */ );
 
@@ -304,9 +305,6 @@ InitAtoms(void);
 extern _X_EXPORT void
 SetVendorRelease(int release);
 
-extern _X_EXPORT void
-SetVendorString(const char *string);
-
 int
 dix_main(int argc, char *argv[], char *envp[]);
 
@@ -424,6 +422,10 @@ DeliverTouchEvents(DeviceIntPtr /* dev */ ,
                    InternalEvent * /* ev */ ,
                    XID /* resource */ );
 
+extern Bool
+DeliverGestureEventToOwner(DeviceIntPtr dev, GestureInfoPtr gi,
+                           InternalEvent *ev);
+
 extern void
 InitializeSprite(DeviceIntPtr /* pDev */ ,
                  WindowPtr /* pWin */ );
@@ -452,6 +454,10 @@ DeliverGrabbedEvent(InternalEvent * /* event */ ,
                     DeviceIntPtr /* thisDev */ ,
                     Bool /* deactivateGrab */ );
 
+extern void
+FreezeThisEventIfNeededForSyncGrab(DeviceIntPtr thisDev,
+                                   InternalEvent *event);
+
 extern void
 FixKeyState(DeviceEvent * /* event */ ,
             DeviceIntPtr /* keybd */ );
@@ -609,6 +615,13 @@ extern Bool
 IsPointerEvent(InternalEvent *event);
 extern Bool
 IsTouchEvent(InternalEvent *event);
+Bool
+IsGestureEvent(InternalEvent *event);
+Bool
+IsGestureBeginEvent(InternalEvent *event);
+Bool
+IsGestureEndEvent(InternalEvent *event);
+
 extern _X_EXPORT Bool
 IsMaster(DeviceIntPtr dev);
 extern _X_EXPORT Bool
